package configuration

import (
	"github.com/r3labs/diff/v3"
)

type ChangeEvent struct {
	Changlog diff.Changelog
}

type EventHook func(event ChangeEvent)

func (s *Settings[C]) OnChange(hook EventHook) *Settings[C] {
	s.hooks.change = append(s.hooks.change, hook)
	return s
}

func (s *Settings[C]) notifyChangeHooks(changelog diff.Changelog) *Settings[C] {
	for _, h := range s.hooks.change {
		h(ChangeEvent{Changlog: changelog})
	}
	return s
}

func (s *Settings[C]) setConfigInternal(config C, lock bool) *Settings[C] {

	var (
		changelog diff.Changelog
		err       error
	)

	if lock {
		s.Lock()
	}

	defer func() {
		if lock {
			s.Unlock()
		}

		if len(changelog) > 0 {
			go s.notifyChangeHooks(changelog)
		}

	}()

	if err := validateConfig[C](config); err != nil {
		s.errors = append(s.errors, err)
		return s
	}

	d, err := diff.NewDiffer()
	if err != nil {
		s.errors = append(s.errors, err)
		return s
	}

	d.ConvertCompatibleTypes = true
	d.AllowTypeMismatch = true

	changelog, err = d.Diff(s.config, config)
	if err != nil {
		s.errors = append(s.errors, err)
		return s
	}

	s.config = config
	return s
}

func (s *Settings[C]) SetConfig(config C) *Settings[C] {
	return s.setConfigInternal(config, true)
}
