package configuration

import (
	"reflect"
)

func getTags(data reflect.Value, tags []string, callback handleTagCallback) {

	for i := 0; i < data.NumField(); i++ {
		field := data.Field(i)

		if field.Kind() == reflect.Struct {
			getTags(field, tags, callback) // recurse
			continue
		}

		for _, tag := range tags {
			tagValue := data.Type().Field(i).Tag.Get(tag)

			if tagValue != "" {
				callback(tagValue, field)
			}
		}
	}

}

type handleTagCallback func(s string, field reflect.Value)

func runOnTags(data any, tags []string, callback handleTagCallback) error {

	t := reflect.TypeOf(data)

	if t.Kind() == reflect.Struct {
		getTags(reflect.ValueOf(data), tags, callback)
		return nil
	}

	if t.Kind() == reflect.Ptr {
		g := reflect.ValueOf(data).Elem()
		t := g.Type()

		if t.Kind() == reflect.Struct {
			getTags(g, tags, callback)
			return nil
		}
	}

	return newUnsupportedReflectKindError(t)

}
