package configuration

import (
	"reflect"
	"strconv"
	"strings"
	"testing"
)

type T2 struct {
	AA string `default:"Hello"`
}

type T1 struct {
	A int    `yaml:"A" json:"A" toml:"A" default:"13" properties:"A"`
	B string `yaml:"B" json:"B" toml:"B" properties:"B"`
	C string `yaml:"B" default:"Hello" json:"B" toml:"B" properties:"B"`
	D bool   `yaml:"B" json:"B" toml:"B" properties:"B" default:"true"`
	E T2     `yaml:"E" json:"E" toml:"E" properties:"E"`
}

func TestRunOnTags(t *testing.T) {
	tt := T1{}

	values := []string{}

	err := runOnTags(tt, []string{"default"}, func(s string, field reflect.Value) {
		values = append(values, s)
	})

	if strings.Join(values, ",") != "13,Hello,true,Hello" {
		t.Error("Expected 13,Hello,true,Hello got ", strings.Join(values, ","))
	}

	if err != nil {
		t.Error(err)
	}
}

func TestRunOnTagsAndSet(t *testing.T) {

	tt := T1{}

	err := runOnTags(&tt, []string{"default"}, func(s string, field reflect.Value) {

		if field.CanSet() {
			switch field.Kind() {
			case reflect.String:
				field.SetString(s)
			case reflect.Int:
				intVar, err := strconv.Atoi(s)
				if err != nil {
					panic(err)
				}
				field.SetInt(int64(intVar))
			case reflect.Bool:
				field.SetBool(s == "true")
			default:
				t.Errorf("Unsupported type %s", field.Kind())
			}

		}
	})

	if err != nil {
		t.Error(err)
	}

	if tt.E.AA != "Hello" {
		t.Error("Expected Hello, got ", tt.E.AA)
	}

	if tt.A != 13 {
		t.Error("Expected 13, got ", tt.A)
	}

	if tt.D != true {
		t.Error("Expected true, got ", tt.D)
	}

}
